@@ -22,52 +22,65 @@
 
 #include <KPluginFactory>
 #include <KAboutData>
-#include <KDebug>
 #include <KMessageBox>
+#include <KLocalizedString>
 
 #include <QHBoxLayout>
+#include <QTimer>
+#include <qstandardpaths.h>
 #include <QLabel>
 
+#include <QQuickView>
+
 #include <kscreen/config.h>
+#include <kscreen/output.h>
+#include <kscreen/getconfigoperation.h>
+#include <kscreen/setconfigoperation.h>
+
+#include <qquickitem.h>
 
 K_PLUGIN_FACTORY(KCMDisplayConfigurationFactory, registerPlugin<KCMKScreen>();)
-K_EXPORT_PLUGIN(KCMDisplayConfigurationFactory ("kcm_kscreen" /* kcm name */,
-                                                "kcm_displayconfiguration" /* catalog name */))
-
-#define QML_PATH "kcm_kscreen/qml/"
 
 using namespace KScreen;
 
-Q_DECLARE_METATYPE(KScreen::Output*)
-Q_DECLARE_METATYPE(KScreen::Screen*)
+Q_DECLARE_METATYPE(KScreen::OutputPtr)
+Q_DECLARE_METATYPE(KScreen::ScreenPtr)
 
-KCMKScreen::KCMKScreen(QWidget* parent, const QVariantList& args) :
-    KCModule(KCMDisplayConfigurationFactory::componentData(), parent, args)
+KCMKScreen::KCMKScreen(QWidget* parent, const QVariantList& args)
+    : KCModule(parent, args)
+    , mKScreenWidget(0)
 {
     setButtons(Apply | Default);
 
     KAboutData* about =
-        new KAboutData("kscreen", "kcm_kscren",
-                    ki18n("Display Configuration"),
-                    "", ki18n("Configuration for displays"),
-                    KAboutData::License_GPL_V2, ki18n("(c), 2012-2013 Daniel Vrátil"));
+        new KAboutData(QStringLiteral("kcm_kscreen"),
+                    i18n("Display Configuration"),
+                    QString(KSCREEN_VERSION), i18n("Configuration for displays"),
+                    KAboutLicense::GPL_V2, i18n("(c), 2012-2013 Daniel Vrátil"));
 
-    about->addAuthor(ki18n("Daniel Vrátil"), ki18n("Maintainer") , "dvratil@redhat.com");
+    about->addAuthor(i18n("Daniel Vrátil"), i18n("Maintainer") , QStringLiteral("dvratil@redhat.com"));
     setAboutData(about);
+}
 
+void KCMKScreen::configReady(ConfigOperation* op)
+{
     QHBoxLayout *layout = new QHBoxLayout(this);
-    if (KScreen::Config::current()) {
-        mKScreenWidget = new Widget(this);
-        layout->addWidget(mKScreenWidget);
 
-        connect(mKScreenWidget, SIGNAL(changed()),
-                this, SLOT(changed()));
-    } else {
+    if (op->hasError()) {
         mKScreenWidget = 0;
+        delete mKScreenWidget;
         QLabel *errorLabel = new QLabel(this);
         layout->addWidget(errorLabel);
         errorLabel->setText(i18n("No kscreen backend found. Please check your kscreen installation."));
+        return;
     }
+
+    mKScreenWidget = new Widget(this);
+    mKScreenWidget->setConfig(qobject_cast<GetConfigOperation*>(op)->config());
+    layout->addWidget(mKScreenWidget);
+
+    connect(mKScreenWidget, SIGNAL(changed()),
+            this, SLOT(changed()));
 }
 
 KCMKScreen::~KCMKScreen()
@@ -76,36 +89,36 @@ KCMKScreen::~KCMKScreen()
 
 void KCMKScreen::save()
 {
-    kDebug() << "Saving";
+    qDebug() << "Saving";
 
     if (!mKScreenWidget) {
         return;
     }
 
-    KScreen::Config *config = mKScreenWidget->currentConfig();
+    const KScreen::ConfigPtr &config = mKScreenWidget->currentConfig();
 
     bool atLeastOneEnabledOutput = false;
-    Q_FOREACH(KScreen::Output *output, config->outputs()) {
-        KScreen::Mode *mode = output->currentMode();
+    Q_FOREACH(const KScreen::OutputPtr &output, config->outputs()) {
+        KScreen::ModePtr mode = output->currentMode();
 
         if (output->isEnabled()) {
             atLeastOneEnabledOutput = true;
         }
 
-        kDebug() << output->name() << "\n"
+        qDebug() << output->name() << output->id() << output.data() << "\n"
                 << "	Connected:" << output->isConnected() << "\n"
                 << "	Enabled:" << output->isEnabled() << "\n"
                 << "	Primary:" << output->isPrimary() << "\n"
                 << "	Rotation:" << output->rotation() << "\n"
-                << "	Mode:" << (mode ? mode->name() : "unknown") << "@" << (mode ? mode->refreshRate() : 0.0) << "Hz" << "\n"
+                << "	Mode:" << (mode ? mode->name() : QStringLiteral("unknown")) << "@" << (mode ? mode->refreshRate() : 0.0) << "Hz" << "\n"
                 << "    Position:" << output->pos().x() << "x" << output->pos().y();
     }
 
     if (!atLeastOneEnabledOutput) {
         if (KMessageBox::warningYesNo(this, i18n("Are you sure you want to disable all outputs?"),
             i18n("Disable all outputs?"),
-            KGuiItem(i18n("&Disable All Outputs"), KIcon(QLatin1String("dialog-ok-apply"))),
-            KGuiItem(i18n("&Reconfigure"), KIcon(QLatin1String("dialog-cancel"))),
+            KGuiItem(i18n("&Disable All Outputs"), QIcon::fromTheme(QLatin1String("dialog-ok-apply"))),
+            KGuiItem(i18n("&Reconfigure"), QIcon::fromTheme(QLatin1String("dialog-cancel"))),
             QString(), KMessageBox::Dangerous) == KMessageBox::No)
         {
             return;
@@ -121,21 +134,20 @@ void KCMKScreen::save()
     }
 
     /* Store the current config, apply settings */
-    config->setConfig(config);
+    new SetConfigOperation(config);
 }
 
 void KCMKScreen::defaults()
 {
+    qDebug() << "LOAD";
     load();
 }
 
 void KCMKScreen::load()
 {
-    if (!mKScreenWidget) {
-        return;
-    }
-
-    mKScreenWidget->setConfig(KScreen::Config::current());
+    qDebug() << "LOAD";
+    connect(new GetConfigOperation(), &GetConfigOperation::finished,
+            this, &KCMKScreen::configReady);
 }
 
 #include "kcm_kscreen.moc"
