@@ -18,12 +18,13 @@
 
 #include "generator.h"
 #include "device.h"
+#include "debug.h"
 
 #include <QDBusReply>
 #include <QDBusMessage>
 #include <QDBusConnection>
+#include <QLoggingCategory>
 
-#include <kdebug.h>
 #include <kscreen/config.h>
 
 Generator* Generator::instance = 0;
@@ -45,6 +46,7 @@ Generator::Generator()
  : QObject()
  , m_forceLaptop(false)
  , m_forceLidClosed(false)
+ , m_forceNotLaptop(false)
  , m_forceDocked(false)
 {
     connect(Device::self(), SIGNAL(ready()), SIGNAL(ready()));
@@ -60,20 +62,24 @@ Generator::~Generator()
 {
 }
 
-KScreen::Config* Generator::idealConfig()
+void Generator::setCurrentConfig(const KScreen::ConfigPtr &currentConfig)
 {
-    KDebug::Block idealBlock("Ideal Config");
-    KScreen::Config* config = KScreen::Config::current();
-    if (!config) {
-        kDebug() << "Unable get current config";
-        return 0;
-    }
+    m_currentConfig = currentConfig;
+}
+
+
+KScreen::ConfigPtr Generator::idealConfig(const KScreen::ConfigPtr &currentConfig)
+{
+    Q_ASSERT(currentConfig);
+
+//     KDebug::Block idealBlock("Ideal Config");
+    KScreen::ConfigPtr config = currentConfig->clone();
 
     disableAllDisconnectedOutputs(config->outputs());
 
     KScreen::OutputList outputs = config->connectedOutputs();
 
-    kDebug() << "Connected outputs: " << outputs.count();
+    qCDebug(KSCREEN_KDED) << "Connected outputs: " << outputs.count();
 
     if (outputs.isEmpty()) {
         return config;
@@ -89,46 +95,47 @@ KScreen::Config* Generator::idealConfig()
         return fallbackIfNeeded(config);
     }
 
-    kDebug() << "Extend to Right";
+    qCDebug(KSCREEN_KDED) << "Extend to Right";
     extendToRight(outputs);
 
     return fallbackIfNeeded(config);
 }
 
-KScreen::Config* Generator::fallbackIfNeeded(KScreen::Config* config)
+KScreen::ConfigPtr Generator::fallbackIfNeeded(const KScreen::ConfigPtr &config)
 {
-    kDebug();
+    qCDebug(KSCREEN_KDED) << "fallbackIfNeeded()";
+
+    KScreen::ConfigPtr newConfig;
+
     //If the ideal config can't be applied, try clonning
     if (!KScreen::Config::canBeApplied(config)) {
-        delete config;
-        if (Device::self()->isLaptop()) {
-            config = displaySwitch(1);// Try to clone at our best
+        if (isLaptop()) {
+            newConfig = displaySwitch(1);// Try to clone at our best
         } else {
-            config = KScreen::Config::current();
+            newConfig = config;
             KScreen::OutputList outputList = config->connectedOutputs();
             outputList.value(outputList.keys().first())->setPrimary(true);
             cloneScreens(outputList);
         }
+    } else {
+        newConfig = config;
     }
 
     //If after trying to clone at our best, we fail... return current
-    if (!KScreen::Config::canBeApplied(config)) {
-        kDebug() << "Can't be applied";
-        delete config;
-        config = KScreen::Config::current();
+    if (!KScreen::Config::canBeApplied(newConfig)) {
+        qCDebug(KSCREEN_KDED) << "Config cannot be applied";
+        newConfig = config;
     }
 
     return config;
 }
 
-KScreen::Config* Generator::displaySwitch(int iteration)
+KScreen::ConfigPtr Generator::displaySwitch(int iteration)
 {
-    KDebug::Block switchBlock("Display Switch");
-    KScreen::Config* config = KScreen::Config::current();
-    if (!config) {
-        kDebug() << "Unable to get current config";
-        return 0;
-    }
+//     KDebug::Block switchBlock("Display Switch");
+    KScreen::ConfigPtr config = m_currentConfig;
+    Q_ASSERT(config);
+
 
     KScreen::OutputList outputs = config->connectedOutputs();
 
@@ -143,21 +150,21 @@ KScreen::Config* Generator::displaySwitch(int iteration)
     }
 
     if (iteration == 1) {
-        kDebug() << "Cloning";
+        qCDebug(KSCREEN_KDED) << "Cloning";
         embeddedOutput(outputs)->setPrimary(true);
         cloneScreens(outputs);
 
         return config;
     }
 
-    KScreen::Output* embedded, *external;
+    KScreen::OutputPtr embedded, external;
     embedded = embeddedOutput(outputs);
     outputs.remove(embedded->id());
     external = outputs.value(outputs.keys().first());
 
 
     if (iteration == 2) {
-        kDebug() << "Extend to left";
+        qCDebug(KSCREEN_KDED) << "Extend to left";
         external->setEnabled(true);
         external->setPos(QPoint(0,0));
         external->setCurrentModeId(external->preferredModeId());
@@ -171,7 +178,7 @@ KScreen::Config* Generator::displaySwitch(int iteration)
     }
 
     if (iteration == 3) {
-        kDebug() << "Turn of embedded (laptop)";
+        qCDebug(KSCREEN_KDED) << "Turn off embedded (laptop)";
         embedded->setEnabled(false);
         embedded->setPrimary(false);
 
@@ -182,7 +189,7 @@ KScreen::Config* Generator::displaySwitch(int iteration)
     }
 
     if (iteration == 4) {
-        kDebug() << "Turn off external screen";
+        qCDebug(KSCREEN_KDED) << "Turn off external screen";
         embedded->setEnabled(true);
         embedded->setPrimary(true);
         embedded->setPos(QPoint(0,0));
@@ -194,7 +201,7 @@ KScreen::Config* Generator::displaySwitch(int iteration)
     }
 
     if (iteration == 5) {
-        kDebug() << "Extend to the right";
+        qCDebug(KSCREEN_KDED) << "Extend to the right";
         embedded->setPos(QPoint(0,0));
         embedded->setCurrentModeId(embedded->preferredModeId());
         embedded->setPrimary(true);
@@ -220,18 +227,18 @@ uint qHash(const QSize &size)
 void Generator::cloneScreens(KScreen::OutputList& outputs)
 {
     if (outputs.isEmpty()) {
-        kWarning() << "output list to clone is empty";
+        qCWarning(KSCREEN_KDED) << "output list to clone is empty";
         return;
     }
 
     QSet<QSize> commonSizes;
-    const QSize maxSize  = KScreen::Config::current()->screen()->maxSize();
+    const QSize maxSize  = m_currentConfig->screen()->maxSize();
 
     QList<QSet<QSize> >modes;
-    Q_FOREACH(KScreen::Output *output, outputs) {
+    Q_FOREACH(const KScreen::OutputPtr &output, outputs) {
         QSet<QSize> modeSizes;
-        KScreen::ModeList modes = output->modes();
-        Q_FOREACH(KScreen::Mode *mode, modes) {
+        const KScreen::ModeList modes = output->modes();
+        Q_FOREACH(const KScreen::ModePtr &mode, modes) {
             const QSize size = mode->size();
             if (size.width() > maxSize.width() || size.height() > maxSize.height()) {
                 continue;
@@ -248,10 +255,10 @@ void Generator::cloneScreens(KScreen::OutputList& outputs)
         commonSizes.intersect(modeSizes);
     }
 
-    kDebug() << "Common sizes: " << commonSizes;
+    qCDebug(KSCREEN_KDED) << "Common sizes: " << commonSizes;
     //fallback to biggestMode if no commonSizes have been found
     if (commonSizes.isEmpty()) {
-        Q_FOREACH(KScreen::Output *output, outputs) {
+        Q_FOREACH(KScreen::OutputPtr output, outputs) {
             output->setEnabled(true);
             output->setPos(QPoint(0, 0));
             output->setCurrentModeId(biggestMode(output->modes())->id());
@@ -266,9 +273,9 @@ void Generator::cloneScreens(KScreen::OutputList& outputs)
     QSize biggestSize = commonSizeList.last();
 
     //Finally, look for the mode with biggestSize and biggest refreshRate and set it
-    kDebug() << "Biggest Size: " << biggestSize;
-    KScreen::Mode* bestMode;
-    Q_FOREACH(KScreen::Output *output, outputs) {
+    qCDebug(KSCREEN_KDED) << "Biggest Size: " << biggestSize;
+    KScreen::ModePtr bestMode;
+    Q_FOREACH(KScreen::OutputPtr output, outputs) {
         bestMode = bestModeForSize(output->modes(), biggestSize);
         output->setEnabled(true);
         output->setPos(QPoint(0, 0));
@@ -276,11 +283,11 @@ void Generator::cloneScreens(KScreen::OutputList& outputs)
     }
 }
 
-void Generator::singleOutput(KScreen::OutputList& outputs)
+void Generator::singleOutput(KScreen::OutputList &outputs)
 {
     Q_ASSERT(!outputs.isEmpty());
 
-    KScreen::Output* output = outputs.take(outputs.keys().first());
+    KScreen::OutputPtr output = outputs.take(outputs.keys().first());
     Q_ASSERT(output);
 
     output->setCurrentModeId(output->preferredModeId());
@@ -289,13 +296,13 @@ void Generator::singleOutput(KScreen::OutputList& outputs)
     output->setPos(QPoint(0,0));
 }
 
-void Generator::laptop(KScreen::OutputList& outputs)
+void Generator::laptop(KScreen::OutputList &outputs)
 {
     Q_ASSERT(!outputs.isEmpty());
 
-    KDebug::Block laptopBlock("Laptop config");
+//     KDebug::Block laptopBlock("Laptop config");
 
-    KScreen::Output* embedded = embeddedOutput(outputs);
+    KScreen::OutputPtr embedded = embeddedOutput(outputs);
     /* Apparently older laptops use "VGA-*" as embedded output ID, so embeddedOutput()
      * will fail, because it looks only for modern "LVDS", "EDP", etc. If we
      * fail to detect which output is embedded, just use the one  with the lowest
@@ -310,17 +317,17 @@ void Generator::laptop(KScreen::OutputList& outputs)
     outputs.remove(embedded->id());
 
     if (outputs.isEmpty()) {
-        kWarning() << "No external outputs found, going for singleOutput()";
+        qCWarning(KSCREEN_KDED) << "No external outputs found, going for singleOutput()";
         outputs.insert(embedded->id(), embedded);
         return singleOutput(outputs);
     }
 
     if (isLidClosed() && outputs.count() == 1) {
-        kDebug() << "With lid closed";
+        qCDebug(KSCREEN_KDED) << "With lid closed";
         embedded->setEnabled(false);
         embedded->setPrimary(false);
 
-        KScreen::Output* external = outputs.value(outputs.keys().first());
+        KScreen::OutputPtr external = outputs.value(outputs.keys().first());
         external->setEnabled(true);
         external->setPrimary(true);
         external->setCurrentModeId(external->preferredModeId());
@@ -330,7 +337,7 @@ void Generator::laptop(KScreen::OutputList& outputs)
     }
 
     if (isLidClosed() && outputs.count() > 1) {
-        kDebug() << "Lid is closed, and more than one output";
+        qCDebug(KSCREEN_KDED) << "Lid is closed, and more than one output";
         embedded->setEnabled(false);
         embedded->setPrimary(false);
 
@@ -338,7 +345,7 @@ void Generator::laptop(KScreen::OutputList& outputs)
         return;
     }
 
-    kDebug() << "Lid is open";
+    qCDebug(KSCREEN_KDED) << "Lid is open";
     //If lid is open, laptop screen shuold be primary
     embedded->setPos(QPoint(0,0));
     embedded->setCurrentModeId(embedded->preferredModeId());
@@ -351,7 +358,7 @@ void Generator::laptop(KScreen::OutputList& outputs)
     } else {
         globalWidth = embedded->preferredMode()->size().height();
     }
-    KScreen::Output* biggest = biggestOutput(outputs);
+    KScreen::OutputPtr biggest = biggestOutput(outputs);
     outputs.remove(biggest->id());
 
     biggest->setPos(QPoint(globalWidth, 0));
@@ -364,7 +371,7 @@ void Generator::laptop(KScreen::OutputList& outputs)
     } else {
         globalWidth += biggest->currentMode()->size().height();
     }
-    Q_FOREACH(KScreen::Output* output, outputs) {
+    Q_FOREACH(KScreen::OutputPtr output, outputs) {
         output->setEnabled(true);
         output->setCurrentModeId(output->preferredModeId());
         output->setPos(QPoint(globalWidth, 0));
@@ -378,18 +385,18 @@ void Generator::laptop(KScreen::OutputList& outputs)
     }
 
     if (isDocked()) {
-        kDebug() << "Docked";
+        qCDebug(KSCREEN_KDED) << "Docked";
         embedded->setPrimary(false);
         biggest->setPrimary(true);
     }
 }
 
-void Generator::extendToRight(KScreen::OutputList& outputs)
+void Generator::extendToRight(KScreen::OutputList &outputs)
 {
     Q_ASSERT(!outputs.isEmpty());
 
-    kDebug() << "Extending to the right";
-    KScreen::Output* biggest = biggestOutput(outputs);
+    qCDebug(KSCREEN_KDED) << "Extending to the right";
+    KScreen::OutputPtr biggest = biggestOutput(outputs);
     Q_ASSERT(biggest);
 
     outputs.remove(biggest->id());
@@ -406,7 +413,7 @@ void Generator::extendToRight(KScreen::OutputList& outputs)
         globalWidth = biggest->currentMode()->size().height();
     }
 
-    Q_FOREACH(KScreen::Output* output, outputs) {
+    Q_FOREACH(KScreen::OutputPtr output, outputs) {
         output->setEnabled(true);
         output->setPrimary(false);
         output->setCurrentModeId(output->preferredModeId());
@@ -420,11 +427,11 @@ void Generator::extendToRight(KScreen::OutputList& outputs)
     }
 }
 
-KScreen::Mode* Generator::biggestMode(const KScreen::ModeList& modes)
+KScreen::ModePtr Generator::biggestMode(const KScreen::ModeList &modes)
 {
     int modeArea, biggestArea = 0;
-    KScreen::Mode* biggestMode = 0;
-    Q_FOREACH(KScreen::Mode* mode, modes) {
+    KScreen::ModePtr biggestMode;
+    Q_FOREACH(const KScreen::ModePtr &mode, modes) {
         modeArea = mode->size().width() * mode->size().height();
         if (modeArea < biggestArea) {
             continue;
@@ -444,10 +451,10 @@ KScreen::Mode* Generator::biggestMode(const KScreen::ModeList& modes)
     return biggestMode;
 }
 
-KScreen::Mode* Generator::bestModeForSize(const KScreen::ModeList& modes, const QSize &size)
+KScreen::ModePtr Generator::bestModeForSize(const KScreen::ModeList &modes, const QSize &size)
 {
-    KScreen::Mode *bestMode = 0;
-    Q_FOREACH(KScreen::Mode *mode, modes) {
+    KScreen::ModePtr bestMode;
+    Q_FOREACH(const KScreen::ModePtr &mode, modes) {
         if (mode->size() != size) {
             continue;
         }
@@ -465,12 +472,12 @@ KScreen::Mode* Generator::bestModeForSize(const KScreen::ModeList& modes, const
     return bestMode;
 }
 
-KScreen::Output* Generator::biggestOutput(const KScreen::OutputList &outputs)
+KScreen::OutputPtr Generator::biggestOutput(const KScreen::OutputList &outputs)
 {
     int area, total = 0;
-    KScreen::Output* biggest = 0;
-    Q_FOREACH(KScreen::Output* output, outputs) {
-        KScreen::Mode* mode = output->preferredMode();
+    KScreen::OutputPtr biggest;
+    Q_FOREACH(const KScreen::OutputPtr &output, outputs) {
+        const KScreen::ModePtr mode = output->preferredMode();
         area = mode->size().width() * mode->size().height();
         if (area <= total) {
             continue;
@@ -483,21 +490,21 @@ KScreen::Output* Generator::biggestOutput(const KScreen::OutputList &outputs)
     return biggest;
 }
 
-void Generator::disableAllDisconnectedOutputs(const KScreen::OutputList& outputs)
+void Generator::disableAllDisconnectedOutputs(const KScreen::OutputList &outputs)
 {
-    KDebug::Block disableBlock("Disabling disconnected screens");
-    Q_FOREACH(KScreen::Output* output, outputs) {
+//     KDebug::Block disableBlock("Disabling disconnected screens");
+    Q_FOREACH(KScreen::OutputPtr output, outputs) {
         if (!output->isConnected()) {
-            kDebug() << output->name() << " Disabled";
+            qCDebug(KSCREEN_KDED) << output->name() << " Disabled";
             output->setEnabled(false);
             output->setPrimary(false);
         }
     }
 }
 
-KScreen::Output* Generator::embeddedOutput(const KScreen::OutputList& outputs)
+KScreen::OutputPtr Generator::embeddedOutput(const KScreen::OutputList &outputs)
 {
-    Q_FOREACH(KScreen::Output* output, outputs) {
+    Q_FOREACH(const KScreen::OutputPtr &output, outputs) {
         if (output->type() != KScreen::Output::Panel) {
             continue;
         }
@@ -505,7 +512,7 @@ KScreen::Output* Generator::embeddedOutput(const KScreen::OutputList& outputs)
         return output;
     }
 
-    return 0;
+    return KScreen::OutputPtr();
 }
 
 bool Generator::isLaptop()
@@ -513,6 +520,9 @@ bool Generator::isLaptop()
     if (m_forceLaptop) {
         return true;
     }
+    if (m_forceNotLaptop) {
+        return false;
+    }
 
     return Device::self()->isLaptop();
 }
@@ -522,6 +532,9 @@ bool Generator::isLidClosed()
     if (m_forceLidClosed) {
         return true;
     }
+    if (m_forceNotLaptop) {
+        return false;
+    }
 
     return Device::self()->isLidClosed();
 }
@@ -550,4 +563,8 @@ void Generator::setForceDocked(bool force)
     m_forceDocked = force;
 }
 
-#include "generator.moc"
+void Generator::setForceNotLaptop(bool force)
+{
+    m_forceNotLaptop = force;
+}
+
