@@ -17,6 +17,7 @@
  *************************************************************************************/
 
 #include "serializer.h"
+#include "debug.h"
 
 #include <QtCore/QStringList>
 #include <QtCore/QCryptographicHash>
@@ -24,75 +25,73 @@
 #include <QtCore/QVariant>
 #include <QtCore/QVariantList>
 #include <QtCore/QVariantMap>
-
-#include <qjson/serializer.h>
-#include <qjson/parser.h>
-
-#include <kdebug.h>
+#include <QtCore/QStandardPaths>
+#include <QJsonDocument>
+#include <QDir>
+#include <QLoggingCategory>
 
 #include <kscreen/config.h>
 #include <kscreen/output.h>
 #include <kscreen/edid.h>
-#include <KStandardDirs>
 
-QString Serializer::currentId()
+QString Serializer::configId(const KScreen::ConfigPtr &currentConfig)
 {
-    KScreen::OutputList outputs = KScreen::Config::current()->outputs();
+    KScreen::OutputList outputs = currentConfig->outputs();
 
     QStringList hashList;
-    Q_FOREACH(const KScreen::Output* output, outputs) {
+    qCDebug(KSCREEN_KDED) << "Calculating config ID for" << currentConfig.data();
+    Q_FOREACH(const KScreen::OutputPtr &output, outputs) {
         if (!output->isConnected()) {
             continue;
         }
 
-        kDebug() << "Part of the Id: " << Serializer::outputId(output);
+        qCDebug(KSCREEN_KDED) << "\tPart of the Id: " << Serializer::outputId(output);
         hashList.insert(0, Serializer::outputId(output));
     }
 
     qSort(hashList.begin(), hashList.end());
 
-    QCryptographicHash hash(QCryptographicHash::Md5);
-    hash.addData(hashList.join(QString()).toAscii());
-    return hash.result().toHex();
+    const QByteArray hash = QCryptographicHash::hash(hashList.join(QString()).toLatin1(),
+                                                     QCryptographicHash::Md5).toHex();
+    qCDebug(KSCREEN_KDED) << "\tConfig ID:" << hash;
+    return hash;
 }
 
-bool Serializer::configExists()
+bool Serializer::configExists(const KScreen::ConfigPtr &config)
 {
-    return Serializer::configExists(Serializer::currentId());
+    return Serializer::configExists(Serializer::configId(config));
 }
 
 bool Serializer::configExists(const QString& id)
 {
-    QString path = KStandardDirs::locateLocal("data", "kscreen/"+id);
+    QString path(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QStringLiteral("/kscreen/") + id);
     return QFile::exists(path);
 }
 
-KScreen::Config* Serializer::config(const QString& id)
+KScreen::ConfigPtr Serializer::config(const KScreen::ConfigPtr &currentConfig, const QString& id)
 {
-    QJson::Parser parser;
-    KScreen::Config* config = KScreen::Config::current();
-    if (!config) {
-        return 0;
-    }
+    KScreen::ConfigPtr config = currentConfig->clone();
+
+    QFile file(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QStringLiteral("/kscreen/") + id);
+    if (!file.open(QIODevice::ReadOnly))
+        return KScreen::ConfigPtr();
 
     KScreen::OutputList outputList = config->outputs();
-    QFile file(KStandardDirs::locateLocal("data", "kscreen/"+id));
-    file.open(QIODevice::ReadOnly);
-
-    QVariantList outputs = parser.parse(file.readAll()).toList();
-    Q_FOREACH(KScreen::Output* output, outputList) {
+    QJsonDocument parser;
+    QVariantList outputs = parser.fromJson(file.readAll()).toVariant().toList();
+    Q_FOREACH(KScreen::OutputPtr output, outputList) {
         if (!output->isConnected() && output->isEnabled()) {
             output->setEnabled(false);
         }
     }
 
     Q_FOREACH(const QVariant &info, outputs) {
-        KScreen::Output* output = Serializer::findOutput(info.toMap());
+        KScreen::OutputPtr output = Serializer::findOutput(config, info.toMap());
         if (!output) {
             continue;
         }
 
-        delete outputList.take(output->id());
+        outputList.remove(output->id());
         outputList.insert(output->id(), output);
     }
     config->setOutputs(outputList);
@@ -100,12 +99,12 @@ KScreen::Config* Serializer::config(const QString& id)
     return config;
 }
 
-bool Serializer::saveConfig(KScreen::Config* config)
+bool Serializer::saveConfig(const KScreen::ConfigPtr &config)
 {
-    KScreen::OutputList outputs = config->outputs();
+    const KScreen::OutputList outputs = config->outputs();
 
     QVariantList outputList;
-    Q_FOREACH(KScreen::Output *output, outputs) {
+    Q_FOREACH(const KScreen::OutputPtr &output, outputs) {
         if (!output->isConnected()) {
             continue;
         }
@@ -123,9 +122,9 @@ bool Serializer::saveConfig(KScreen::Config* config)
         info["pos"] = pos;
 
         if (output->isEnabled()) {
-            KScreen::Mode *mode = output->currentMode();
+            const KScreen::ModePtr mode = output->currentMode();
             if (!mode) {
-                kWarning() << "CurrentMode is null" << output->name();
+                qWarning() << "CurrentMode is null" << output->name();
                 return false;
             }
 
@@ -145,28 +144,23 @@ bool Serializer::saveConfig(KScreen::Config* config)
         outputList.append(info);
     }
 
-    QJson::Serializer serializer;
-    QByteArray json = serializer.serialize(outputList);
+    const QString directory = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QStringLiteral("/kscreen/");
+    bool b = QDir().mkpath(directory);
+    Q_ASSERT(b);
+    QString filePath = directory + Serializer::configId(config);
+    QFile file(filePath);
+    b = file.open(QIODevice::WriteOnly);
+    Q_ASSERT(b);
+    file.write(QJsonDocument::fromVariant(outputList).toJson());
+    qCDebug(KSCREEN_KDED) << "Config saved on: " << filePath;
 
-    QString path = KStandardDirs::locateLocal("data", "kscreen/"+ Serializer::currentId());
-    QFile file(path);
-    file.open(QIODevice::WriteOnly);
-    file.write(json);
-    file.close();
-
-    kDebug() << "Config saved on: " << path;
     return true;
 }
 
-KScreen::Output* Serializer::findOutput(const QVariantMap& info)
+KScreen::OutputPtr Serializer::findOutput(const KScreen::ConfigPtr &config, const QVariantMap& info)
 {
-    KScreen::Config *config = KScreen::Config::current();
-    if (!config) {
-        return 0;
-    }
-
     KScreen::OutputList outputs = config->outputs();
-    Q_FOREACH(KScreen::Output* output, outputs) {
+    Q_FOREACH(KScreen::OutputPtr output, outputs) {
         if (!output->isConnected()) {
             continue;
         }
@@ -174,41 +168,39 @@ KScreen::Output* Serializer::findOutput(const QVariantMap& info)
             continue;
         }
 
-        QVariantMap posInfo = info["pos"].toMap();
+        const QVariantMap posInfo = info["pos"].toMap();
         QPoint point(posInfo["x"].toInt(), posInfo["y"].toInt());
         output->setPos(point);
         output->setPrimary(info["primary"].toBool());
         output->setEnabled(info["enabled"].toBool());
         output->setRotation(static_cast<KScreen::Output::Rotation>(info["rotation"].toInt()));
 
-        QVariantMap modeInfo = info["mode"].toMap();
-        QVariantMap modeSize = modeInfo["size"].toMap();
-        QSize size(modeSize["width"].toInt(), modeSize["height"].toInt());
+        const QVariantMap modeInfo = info["mode"].toMap();
+        const QVariantMap modeSize = modeInfo["size"].toMap();
+        const QSize size = QSize(modeSize["width"].toInt(), modeSize["height"].toInt());
 
-        kDebug() << "Finding a mode with: ";
-        kDebug() << size;
-        kDebug() << modeInfo["refresh"].toString();
+        qCDebug(KSCREEN_KDED) << "Finding a mode for" << size << "@" << modeInfo["refresh"].toFloat();
 
         KScreen::ModeList modes = output->modes();
-        Q_FOREACH(KScreen::Mode* mode, modes) {
+        Q_FOREACH(const KScreen::ModePtr &mode, modes) {
             if (mode->size() != size) {
                 continue;
             }
-            if (QString::number(mode->refreshRate()) != modeInfo["refresh"].toString()) {
+            if (!qFuzzyCompare(mode->refreshRate(), modeInfo["refresh"].toFloat())) {
                 continue;
             }
 
-            kDebug() << "Found: " << mode->id() << " " << mode->name();
+            qCDebug(KSCREEN_KDED) << "\tFound: " << mode->id() << " " << mode->size() << "@" << mode->refreshRate();
             output->setCurrentModeId(mode->id());
             break;
         }
         return output;
     }
 
-    return 0;
+    return KScreen::OutputPtr();
 }
 
-QString Serializer::outputId(const KScreen::Output* output)
+QString Serializer::outputId(const KScreen::OutputPtr &output)
 {
     if (output->edid() && output->edid()->isValid()) {
         return output->edid()->hash();
@@ -217,14 +209,14 @@ QString Serializer::outputId(const KScreen::Output* output)
     return output->name();
 }
 
-QVariantMap Serializer::metadata(const KScreen::Output* output)
+QVariantMap Serializer::metadata(const KScreen::OutputPtr &output)
 {
     QVariantMap metadata;
-    metadata["name"] = output->name();
+    metadata[QStringLiteral("name")] = output->name();
     if (!output->edid() || !output->edid()->isValid()) {
         return metadata;
     }
 
-    metadata["fullname"] = output->edid()->deviceId();
+    metadata[QStringLiteral("fullname")] = output->edid()->deviceId();
     return metadata;
 }
